@@ -24,7 +24,10 @@ use AI::MXNet::Module;
 use AI::MXNet::Module::Bucketing;
 use AI::MXNet::RNN;
 use AI::MXNet::Visualization;
-our $VERSION = '0.04';
+use AI::MXNet::RecordIO;
+use AI::MXNet::Image;
+use AI::MXNet::Contrib;
+our $VERSION = '0.95';
 
 sub import
 {
@@ -36,6 +39,7 @@ sub import
         {
             my $short_name_package =<<"EOP";
             package $short_name;
+            no warnings 'redefine';
             sub nd { 'AI::MXNet::NDArray' }
             sub sym { 'AI::MXNet::Symbol' }
             sub symbol { 'AI::MXNet::Symbol' }
@@ -45,15 +49,24 @@ sub import
             sub opt { 'AI::MXNet::Optimizer' }
             sub rnd { 'AI::MXNet::Random' }
             sub random { 'AI::MXNet::Random' }
+            sub Context { shift; AI::MXNet::Context->new(\@_) }
             sub cpu { AI::MXNet::Context->cpu(\$_[1]//0) }
             sub gpu { AI::MXNet::Context->gpu(\$_[1]//0) }
             sub kv { 'AI::MXNet::KVStore' }
+            sub recordio { 'AI::MXNet::RecordIO' }
             sub io { 'AI::MXNet::IO' }
             sub metric { 'AI::MXNet::Metric' }
             sub mod { 'AI::MXNet::Module' }
             sub viz { 'AI::MXNet::Visualization' }
             sub rnn { 'AI::MXNet::RNN' }
             sub callback { 'AI::MXNet::Callback' }
+            sub img { 'AI::MXNet::Image' }
+            sub contrib { 'AI::MXNet::Contrib' }
+            sub AttrScope { shift; AI::MXNet::Symbol::AttrScope->new(\@_) }
+            *AI::MXNet::Symbol::AttrScope::current = sub { \$${short_name}::AttrScope; };
+            \$${short_name}::AttrScope = AI::MXNet::Symbol::AttrScope->new;
+            *AI::MXNet::Context::current_ctx = sub { \$${short_name}::Context; };
+            \$${short_name}::Context = AI::MXNet::Context->new(device_type => 'cpu', device_id => 0);
             1;
 EOP
             eval $short_name_package;
@@ -131,13 +144,12 @@ AI::MXNet - Perl interface to MXNet machine learning library
 
 =head1 DESCRIPTION
 
-    Perl interface to MXNet machine learning library.
+Perl interface to MXNet machine learning library.
 
 =head1 BUGS AND INCOMPATIBILITIES
 
-Parity with Python inteface is not yet achieved.
-Pod mostly contains Python documentation taken as is.
-This is WIP.
+Parity with Python inteface is mostly achieved, few deprecated
+and not often used features left unported for now.
 
 =head1 SEE ALSO
 
