@@ -5,9 +5,9 @@ use AI::MXNet::Base;
 use AI::MXNet::Function::Parameters;
 use Scalar::Util qw/blessed/;
 
-=head1 DESCRIPTION
+=head1 NAME
 
-    NDArray interface of mxnet.
+AI::MXNet::IO - NDArray interface of mxnet.
 =cut
 
 # Convert data into canonical form.
@@ -59,6 +59,9 @@ method init_data(
     return \@ret;
 }
 
+method DataDesc(@args)  { AI::MXNet::DataDesc->new(@args)  }
+method DataBatch(@args) { AI::MXNet::DataBatch->new(@args) }
+
 package AI::MXNet::DataDesc;
 use Mouse;
 use overload '""'  => \&stringify,
@@ -71,14 +74,19 @@ has 'layout' => (is => 'ro', isa => "Str",   default => 'NCHW');
 around BUILDARGS => sub {
     my $orig  = shift;
     my $class = shift;
-    return $class->$orig(name => $_[0], shape => $_[1]) if @_ == 2;
+    if(@_ >= 2 and ref $_[1] eq 'ARRAY')
+    {
+        my $name  = shift;
+        my $shape = shift;
+        return $class->$orig(name => $name, shape => $shape, @_);
+    }
     return $class->$orig(@_);
 };
 
 method stringify($other=, $reverse=)
 {
     sprintf(
-        "DataDesc[%s,%s,%s,%s]", 
+        "DataDesc[%s,%s,%s,%s]",
         $self->name,
         join('x', @{ $self->shape }),
         $self->dtype,
@@ -91,21 +99,26 @@ method to_nameshape($other=, $reverse=)
     [$self->name, $self->shape];
 }
 
+=head1 NAME
+
+AI::MXNet::DataDesc - A container class for describing the data layout.
+=cut
+
 =head2 get_batch_axis
 
-        Get the dimension that corresponds to the batch size.
+Get the dimension that corresponds to the batch size.
 
-        Parameters
-        ----------
-        layout : str
-            layout string. For example, "NCHW".
+Parameters
+----------
+layout : str
+    layout string. For example, "NCHW".
 
-        Returns
-        -------
-        An axis indicating the batch_size dimension. When data-parallelism is
-        used, the data will be automatically split and concatenate along the batch_size
-        dimension. Axis can be -1, which means the whole array will be copied for each
-        data-parallelism device.
+Returns
+-------
+An axis indicating the batch_size dimension. When data-parallelism is
+used, the data will be automatically split and concatenate along the batch_size
+dimension. Axis can be -1, which means the whole array will be copied for each
+data-parallelism device.
 =cut
 
 method get_batch_axis(Str|Undef $layout)
@@ -116,15 +129,15 @@ method get_batch_axis(Str|Undef $layout)
 
 =head2 get_list
 
-        Get DataDesc list from attribute lists.
+Coverts the input to an array ref AI::MXNet::DataDesc objects.
 
-        Parameters
-        ----------
-        shapes : hashref with (name, shape)
-        types :  hashref with (name, type)
+Parameters
+----------
+$shapes : HashRef[Shape]
+$types :  Maybe[HashRef[Dtype]]
 =cut
 
-method get_list(HashRef[Shape] $shapes, HashRef[Dtype]|Undef $types)
+method get_list(HashRef[Shape] $shapes, Maybe[HashRef[Dtype]] $types=)
 {
     $types //= {};
     return [
@@ -141,9 +154,14 @@ method get_list(HashRef[Shape] $shapes, HashRef[Dtype]|Undef $types)
 package AI::MXNet::DataBatch;
 use Mouse;
 
+=head1 NAME
+
+AI::MXNet::DataBatch - A container for a mini-batch of the data and related information.
+=cut
+
 =head1 DESCRIPTION
 
-    Default object for holding a mini-batch of data and related information.
+Default object for holding a mini-batch of data and related information.
 =cut
 
 has 'data'          => (is => 'rw', isa => 'Maybe[ArrayRef[AI::MXNet::NDArray]]', required => 1);
@@ -159,23 +177,23 @@ use Mouse;
 use overload '<>' =>  sub { shift->next },
              '@{}' => sub { shift->list };
 
-=head2 DESCRIPTION
+=head1 NAME
 
-    DataIter object in mxnet.
+AI::MXNet::DataIter - A parent class for MXNet data iterators.
 =cut
 
 has 'batch_size' => (is => 'rw', isa => 'Int', default => 0);
 
 =head2 reset
 
-        Reset the iterator.
+Reset the iterator.
 =cut
 
 method reset(){}
 
 =head2 list
 
-        Returns remaining iterator items as array ref.
+Returns remaining iterator items as an array ref.
 =cut
 
 method list()
@@ -190,14 +208,12 @@ method list()
 
 =head2 next
 
-        Get next data batch from iterator. Equivalent to
-        self.iter_next()
-        DataBatch(self.getdata(), self.getlabel(), self.getpad(), None)
+Returns the next data batch from the iterator.
 
-        Returns
-        -------
-        data : DataBatch
-            The data of next batch.
+Returns
+-------
+$data : AI::MXNet::DataBatch
+The data of next batch.
 =cut
 
 method next()
@@ -219,60 +235,55 @@ method next()
 
 =head2 iter_next
 
-        Iterate to next batch.
+Iterate to next batch.
 
-        Returns
-        -------
-        has_next : boolean
-            Whether the move is successful.
+Returns
+-------
+$has_next : Bool
 =cut
 
 method iter_next(){}
 
 =head2 get_data
 
-        Get data of current batch.
+The data of current batch.
 
-        Returns
-        -------
-        data : NDArray
-            The data of current batch.
+Returns
+-------
+data : AI::MXNet::NDArray
 =cut
 
 method get_data(){}
 
 =head2 getlabel
 
-        Get data of current batch.
+The label of the current batch.
 
-        Returns
-        -------
-        label : NDArray
-            The label of current batch.
+Returns
+-------
+label : AI::MXNet::NDArray
 =cut
 
 method getlabel(){}
 
 =head2 getindex
 
-        Get index of the current batch.
+The index of the current batch.
 
-        Returns
-        -------
-        index : numpy.array
-            The index of current batch
+Returns
+-------
+$index : PDL
 =cut
 
 method getindex(){}
 
 =head2 getpad
 
-        Get the number of padding examples in current batch.
+The number of padding examples in the current batch.
 
-        Returns
-        -------
-        pad : int
-            Number of padding examples in current batch
+Returns
+-------
+$pad : Int
 =cut
 
 method getpad(){}
@@ -282,18 +293,23 @@ use Mouse;
 
 extends 'AI::MXNet::DataIter';
 
-=head2 new
+=head1 NAME
 
-    "Resize a DataIter to given number of batches per epoch.
-    May produce incomplete batch in the middle of an epoch due
-    to padding from internal iterator.
+AI::MXNet::ResizeIter
+=cut
 
-    Parameters
-    ----------
-    data_iter : DataIter
-        Internal data iterator.
-    size : number of batches per epoch to resize to.
-    reset_internal : whether to reset internal iterator on ResizeIter.reset
+=head1 DESCRIPTION
+
+Resize a DataIter to given number of batches per epoch.
+May produce incomplete batch in the middle of an epoch due
+to padding from internal iterator.
+
+Parameters
+----------
+data_iter : DataIter
+    Internal data iterator.
+size : number of batches per epoch to resize to.
+reset_internal : whether to reset internal iterator on ResizeIter.reset
 =cut
 
 has 'data_iter'      => (is => 'ro', isa => 'AI::MXnet::DataIter', required => 1);
@@ -327,7 +343,7 @@ method reset()
     }
 }
 
-method iter_next
+method iter_next()
 {
     return 0 if($self->cur == $self->size);
     $self->current_batch($self->data_iter->next);
@@ -366,33 +382,38 @@ use AI::MXNet::Base;
 use List::Util qw(shuffle);
 extends 'AI::MXNet::DataIter';
 
-=head1 DESCRIPTION
+=head1 NAME
 
-    NDArrayIter object in mxnet. Taking NDArray or numpy array to get dataiter.
-
-    Parameters
-    ----------
-    data: NDArray or numpy.ndarray, a list of them, or a dict of string to them.
-        NDArrayIter supports single or multiple data and label.
-    label: NDArray or numpy.ndarray, a list of them, or a dict of them.
-        Same as data, but is not fed to the model during testing.
-    batch_size: int
-        Batch Size
-    shuffle: bool
-        Whether to shuffle the data
-    last_batch_handle: 'pad', 'discard' or 'roll_over'
-        How to handle the last batch
-
-    Note
-    ----
-    This iterator will pad, discard or roll over the last batch if
-    the size of data does not match batch_size. Roll over is intended
-    for training and can cause problems if used for prediction.
+AI::MXNet::NDArrayIter - Predefined NDArray iterator.
 =cut
 
-has 'data'                => (is => 'rw', isa => 'AcceptableInput|HashRef[AcceptableInput]|ArrayRef[AcceptableInput]|Undef');
+=head1 DESCRIPTION
+
+Predefined NDArray iterator. Accepts PDL or AI::MXNet::NDArray object as an input.
+
+Parameters
+----------
+data: Maybe[AcceptableInput|HashRef[AcceptableInput]|ArrayRef[AcceptableInput]].
+    NDArrayIter supports single or multiple data and label.
+label: Maybe[AcceptableInput|HashRef[AcceptableInput]|ArrayRef[AcceptableInput]].
+    Same as data, but is not given to the model during testing.
+batch_size=1: Int
+    Batch Size
+shuffle=0: Bool
+    Whether to shuffle the data
+last_batch_handle='pad': 'pad', 'discard' or 'roll_over'
+    How to handle the last batch
+
+Note
+----
+This iterator will pad, discard or roll over the last batch if
+the size of data does not match batch_size. Roll over is intended
+for training and can cause problems if used for prediction.
+=cut
+
+has 'data'                => (is => 'rw', isa => 'Maybe[AcceptableInput|HashRef[AcceptableInput]|ArrayRef[AcceptableInput]]');
 has 'data_list'           => (is => 'rw', isa => 'ArrayRef[AI::MXNet::NDArray]');
-has 'label'               => (is => 'rw', isa => 'AcceptableInput|HashRef[AcceptableInput]|ArrayRef[AcceptableInput]|Undef');
+has 'label'               => (is => 'rw', isa => 'Maybe[AcceptableInput|HashRef[AcceptableInput]|ArrayRef[AcceptableInput]]');
 has 'batch_size'          => (is => 'rw', isa => 'Int', default => 1);
 has '_shuffle'            => (is => 'rw', init_arg => 'shuffle', isa => 'Bool', default => 0);
 has 'last_batch_handle'   => (is => 'rw', isa => 'Str', default => 'pad');
@@ -401,6 +422,17 @@ has 'num_source'          => (is => 'rw', isa => 'Int');
 has 'cursor'              => (is => 'rw', isa => 'Int');
 has 'num_data'            => (is => 'rw', isa => 'Int');
 
+around BUILDARGS => sub {
+    my $orig  = shift;
+    my $class = shift;
+    if(@_%2)
+    {
+        my $data  = shift;
+        return $class->$orig(data => $data, @_);
+    }
+    return $class->$orig(@_);
+};
+
 sub BUILD
 {
     my $self  = shift;
@@ -468,7 +500,7 @@ method reset()
     }
     else
     {
-        $self->cursor = -$self->batch_size;
+        $self->cursor(-$self->batch_size);
     }
 }
 
@@ -554,14 +586,9 @@ use AI::MXNet::Base;
 
 extends 'AI::MXNet::DataIter';
 
-=head1 DESCRIPTION
+=head1 NAME
 
-    DataIter built in MXNet. List all the needed functions here.
-
-    Parameters
-    ----------
-    handle : DataIterHandle
-        the handle to the underlying C++ Data Iterator
+AI::MXNet::MXDataIter - A data iterator pre-built in C++ layer of MXNet.
 =cut
 
 has 'handle'           => (is => 'ro', isa => 'DataIterHandle', required => 1);
@@ -604,17 +631,17 @@ sub DEMOLISH
 
 =head2 debug_skip_load
 
-        Set the iterator to simply return always first batch.
-        Notes
-        -----
-        This can be used to test the speed of network without taking
-        the loading delay into account.
+Set the iterator to simply return always first batch.
+Notes
+-----
+This can be used to test the speed of network without taking
+the loading delay into account.
 =cut
 
 method debug_skip_load()
 {
     $self->_debug_skip_load(1);
-    ## TODO logging.info('Set debug_skip_load to be true, will simply return first batch')
+    AI::MXNet::Logging->info('Set debug_skip_load to be true, will simply return first batch');
 }
 
 method reset()
@@ -697,7 +724,7 @@ package AI::MXNet::IO;
 sub NDArrayIter { shift; return AI::MXNet::NDArrayIter->new(@_); }
 
 my %iter_meta;
-method get_iter_meta
+method get_iter_meta()
 {
     return \%iter_meta;
 }
@@ -717,23 +744,6 @@ func _make_io_iterator($handle)
                   ."-------\n"
                   ."iterator: DataIter\n"
                   ."    The result iterator.";
-
-=head2 $iter
-
-        Create an iterator.
-        The parameters listed below can be passed in as keyword arguments.
-
-        Parameters
-        ----------
-        name : string, required.
-            Name of the resulting data iterator.
-
-        Returns
-        -------
-        dataiter: Dataiter
-            the resulting data iterator
-=cut
-
     my $iter = sub {
         my $class = shift;
         my (@args, %kwargs);
