@@ -4,21 +4,16 @@ use AI::MXNet::Function::Parameters;
 
 =head1 NAME
 
-    AI::MXNet::InitDesc
-
-=head1 DESCRIPTION
-
-     Descriptor for initialization pattern.
-=cut
+AI::MXNet::InitDesc - A container for the initialization pattern serialization.
 
 =head2 new
 
-    Parameters
-    ---------
-    name : str
-        name of variable
-    attrs : hash ref of str to str
-        attributes of this variable taken from AI::MXNet::Symbol->attr_dict
+Parameters
+---------
+name : str
+    name of variable
+attrs : hash ref of str to str
+    attributes of this variable taken from AI::MXNet::Symbol->attr_dict
 =cut
 has 'name'   => (is => 'ro', isa => 'Str', required => 1);
 has 'attrs'  => (is => 'rw', isa => 'HashRef[Str]', lazy => 1, default => sub { +{} });
@@ -50,16 +45,11 @@ has 'kwargs' => (is => 'rw', init_arg => undef, isa => 'HashRef');
 
 =head1 NAME
 
-    AI::MXNet::Initializer
-
-=head1 DESCRIPTION
-
-     Base class for all Initializers
-=cut
+AI::MXNet::Initializer - Base class for all Initializers
 
 =head2 register
 
-    Register initializer class to the initializer factory
+Register an initializer class to the AI::MXNet::Initializer factory
 =cut
 
 my %init_registry;
@@ -92,16 +82,15 @@ method register()
 
 =head2 init
 
-        Parameters
-        ----------
-        desc : AI::MXNet::InitDesc|str
-            name of corresponding ndarray
-            object that describes the initializer
+Parameters
+----------
+desc : AI::MXNet::InitDesc|str
+    a name of corresponding ndarray
+    or the object that describes the initializer
 
-        arr : NDArray
-            ndarray to be Initialized
+arr : AI::MXNet::NDArray
+    an ndarray to be Initialized
 =cut
-use Data::Dumper;
 method call(Str|AI::MXNet::InitDesc $desc, AI::MXNet::NDArray $arr)
 {
     return $self->_legacy_init($desc, $arr) unless blessed $desc;
@@ -253,18 +242,20 @@ method _init_default($name, $arr)
     );
 }
 
-=head2
+=head1 NAME
 
-    Initialize by loading pretrained param from file or dict
+AI::MXNet::Load  - Initialize by loading a pretrained param from a hash ref
+=cut
 
-    Parameters
-    ----------
-    param: str or dict of str->NDArray
-        param file or dict mapping name to NDArray.
-    default_init: Initializer
-        default initializer when name is not found in param.
-    verbose: bool
-        log source when initializing.
+=head2 new
+
+Parameters
+----------
+param: HashRef[AI::MXNet::NDArray]
+default_init: Initializer
+    default initializer when a name is not found in the param hash ref.
+verbose: bool
+log the names when initializing.
 =cut
 
 package AI::MXNet::Load;
@@ -314,15 +305,17 @@ method call(Str $name, AI::MXNet::NDArray $arr)
 
 *slice = *call;
 
-=begin
-    Initialize with mixed Initializer
+=head1 NAME
 
-    Parameters
-    ----------
-    patterns: list of str
-        list of regular expression patterns to match parameter names.
-    initializers: list of Initializer
-        list of Initializer corrosponding to patterns
+AI::MXNet::Mixed - A container for multiple initializer patterns.
+=cut
+
+=head2 new
+
+patterns: array ref of str
+    array ref of regular expression patterns to match parameter names.
+initializers: array ref of AI::MXNet::Initializer objects.
+    array ref of Initializers corresponding to the patterns.
 =cut
 
 package AI::MXNet::Mixed;
@@ -397,14 +390,19 @@ method _init_weight(Str $name, AI::MXNet::NDArray $arr)
 
 __PACKAGE__->register;
 
-=begin
+=head1 NAME
 
-    Initialize the weight with uniform [-scale, scale]
+AI::MXNet::Uniform - Initialize the weight with uniform random values
+=cut
 
-    Parameters
-    ----------
-    scale : float, optional
-        The scale of uniform distribution
+=head1 DESCRIPTION
+
+Initialize the weight with uniform random values contained within of [-scale, scale]
+
+Parameters
+----------
+scale : float, optional
+    The scale of the uniform distribution.
 =cut
 
 package AI::MXNet::Uniform;
@@ -424,14 +422,20 @@ method _init_weight(Str $name, AI::MXNet::NDArray $arr)
 }
 
 __PACKAGE__->register;
-=begin
 
-    Initialize the weight with normal(0, sigma)
+=head1 NAME
 
-    Parameters
-    ----------
-    sigma : float, optional
-        Standard deviation for gaussian distribution.
+AI::MXNet::Normal - Initialize the weight with gaussian random values.
+=cut
+
+=head1 DESCRIPTION
+
+Initialize the weight with gaussian random values contained within of [0, sigma]
+
+Parameters
+----------
+sigma : float, optional
+    Standard deviation for the gaussian distribution.
 =cut
 
 package AI::MXNet::Normal;
@@ -452,21 +456,27 @@ method _init_weight(Str $name, AI::MXNet::NDArray $arr)
 
 __PACKAGE__->register;
 
-=begin
-    Intialize weight as Orthogonal matrix
+=head1 NAME
 
-    Parameters
-    ----------
-    scale : float optional
-        scaling factor of weight
+AI::MXNet::Orthogonal - Intialize the weight as an Orthogonal matrix.
+=cut
 
-    rand_type: string optional
-        use "uniform" or "normal" random number to initialize weight
+=head1 DESCRIPTION
 
-    Reference
-    ---------
-    Exact solutions to the nonlinear dynamics of learning in deep linear neural networks
-    arXiv preprint arXiv:1312.6120 (2013).
+Intialize weight as Orthogonal matrix
+
+Parameters
+----------
+scale : float, optional
+    scaling factor of weight
+
+rand_type: string optional
+    use "uniform" or "normal" random number to initialize weight
+
+Reference
+---------
+Exact solutions to the nonlinear dynamics of learning in deep linear neural networks
+arXiv preprint arXiv:1312.6120 (2013).
 =cut
 
 package AI::MXNet::Orthogonal;
@@ -509,20 +519,21 @@ method _init_weight(Str $name, AI::MXNet::NDArray $arr)
 *slice = *call;
 __PACKAGE__->register;
 
-=begin
+=head1 NAME
 
-    Initialize the weight with Xavier or similar initialization scheme.
+AI::MXNet::Xavier - Initialize the weight with Xavier or similar initialization scheme.
+=cut
 
-    Parameters
-    ----------
-    rnd_type: str, optional
-        Use ```gaussian``` or ```uniform``` to init
+=head1 DESCRIPTION
 
-    factor_type: str, optional
-        Use ```avg```, ```in```, or ```out``` to init
-
-    magnitude: float, optional
-        scale of random number range
+Parameters
+----------
+rnd_type: str, optional
+    Use gaussian or uniform.
+factor_type: str, optional
+    Use avg, in, or out.
+magnitude: float, optional
+    The scale of the random number range.
 =cut
 
 package AI::MXNet::Xavier;
@@ -567,18 +578,22 @@ method _init_weight(Str $name, AI::MXNet::NDArray $arr)
 }
 __PACKAGE__->register;
 
-=begin
+=head1 NAME
 
-    Initialize the weight with initialization scheme from
-        Delving Deep into Rectifiers: Surpassing Human-Level Performance on ImageNet Classification.
+AI::MXNet::MSRAPrelu - Custom initialization scheme.
+=cut
 
-    Parameters
-    ----------
-    factor_type: str, optional
-        Use ```avg```, ```in```, or ```out``` to init
+=head1 DESCRIPTION
 
-    slope: float, optional
-        initial slope of any PReLU (or similar) nonlinearities.
+Initialize the weight with initialization scheme from
+Delving Deep into Rectifiers: Surpassing Human-Level Performance on ImageNet Classification.
+
+Parameters
+----------
+factor_type: str, optional
+    Use avg, in, or out.
+slope: float, optional
+    initial slope of any PReLU (or similar) nonlinearities.
 =cut
 
 package AI::MXNet::MSRAPrelu;
@@ -625,6 +640,39 @@ method _init_weight($name, $arr)
 
 __PACKAGE__->register;
 
+package AI::MXNet::LSTMBias;
+
+=head1 NAME
+
+AI::MXNet::LSTMBias - Custom initializer for LSTM cells.
+=cut
+
+=head1 DESCRIPTION
+
+Initializes all biases of an LSTMCell to 0.0 except for
+the forget gate's bias that is set to a custom value.
+
+Parameters
+----------
+forget_bias: float,a bias for the forget gate.
+Jozefowicz et al. 2015 recommends setting this to 1.0.
+=cut
+
+use Mouse;
+extends 'AI::MXNet::Initializer';
+has 'forget_bias' => (is => 'ro', isa => 'Num', required => 1);
+
+method _init_weight(Str $name, AI::MXNet::NDArray $arr)
+{
+    $arr .= 0;
+    # in the case of LSTMCell the forget gate is the second
+    # gate of the 4 LSTM gates, we modify the according values.
+    my $num_hidden = int($arr->shape->[0] / 4);
+    $arr->slice([$num_hidden, 2*$num_hidden-1]) .= $self->forget_bias;
+}
+
+__PACKAGE__->register;
+
 package AI::MXNet::FusedRNN;
 use Mouse;
 use JSON::PP;
@@ -632,27 +680,31 @@ extends 'AI::MXNet::Initializer';
 
 =head1 NAME
 
-AI::MXNet::FusedRNN
+AI::MXNet::FusedRNN - Custom initializer for fused RNN cells.
+=cut
 
 =head1 DESCRIPTION
 
-    Initialze parameters for fused rnn layer
+Initializes parameters for fused rnn layer
 
-    Parameters
-    ----------
-    init : Initializer
-        intializer applied to unpacked weights.
-    num_hidden : int
-        should be the same with arguments passed to FusedRNNCell.
-    num_layers : int
-        should be the same with arguments passed to FusedRNNCell.
-    mode : str
-        should be the same with arguments passed to FusedRNNCell.
-    bidirectional : bool
-        should be the same with arguments passed to FusedRNNCell.
+Parameters
+----------
+init : Initializer
+    intializer applied to unpacked weights.
+num_hidden : int
+    should be the same with arguments passed to FusedRNNCell.
+num_layers : int
+    should be the same with arguments passed to FusedRNNCell.
+mode : str
+    should be the same with arguments passed to FusedRNNCell.
+bidirectional : bool
+    should be the same with arguments passed to FusedRNNCell.
+forget_bias : float
+    should be the same with arguments passed to FusedRNNCell.
 =cut
 
 has 'init'          => (is => 'rw', isa => 'Str|AI::MXNet::Initializer', required => 1);
+has 'forget_bias'   => (is => 'ro', isa => 'Num', default => 1);
 has [qw/num_hidden
        num_layers/] => (is => 'ro', isa => 'Int', required => 1);
 has 'mode'          => (is => 'ro', isa => 'Str', required => 1);
@@ -679,17 +731,29 @@ method _init_weight($name, $arr)
         num_layers    => $self->num_layers,
         mode          => $self->mode,
         bidirectional => $self->bidirectional,
+        forget_bias   => $self->forget_bias,
         prefix        => ''
     );
 
     my $args = $cell->unpack_weights({ parameters => $arr });
     for my $name (keys %{ $args })
     {
-       my $desc = AI::MXNet::InitDesc->new(name => $name);
-       &{$self->init}($desc, $args->{name});
+        my $desc = AI::MXNet::InitDesc->new(name => $name);
+        # for lstm bias, we use a custom initializer
+        # which adds a bias to the forget gate
+        if($self->_mode eq 'lstm' and $name =~ /f_bias$/)
+        {
+            $args->{$name} .= $self->forget_bias;
+        }
+        else
+        {
+            &{$self->init}($desc, $args->{$name});
+        }
     }
 
     $arr .= $cell->pack_weights($args)->{parameters};
 }
 
+__PACKAGE__->register;
+
 1;
