@@ -4,9 +4,9 @@ use warnings;
 use AI::MXNet::Function::Parameters;
 use Scalar::Util qw/blessed/;
 
-=head1 DESCRIPTION
+=head1 NAME
 
-Online evaluation metric module.
+AI::MXNet::Metric - Online evaluation metric module.
 =cut
 
 # Check to see if the two arrays are the same size.
@@ -50,7 +50,12 @@ func check_label_shapes(
 
 package AI::MXNet::EvalMetric;
 use Mouse;
-
+use overload '""' => sub {
+    return "EvalMetric: "
+            .Data::Dumper->new(
+                [shift->get_name_value()]
+            )->Purity(1)->Deepcopy(1)->Terse(1)->Dump
+},  fallback => 1;
 has 'name'       => (is => 'rw', isa => 'Str');
 has 'num'        => (is => 'rw', isa => 'Int');
 has 'num_inst'   => (is => 'rw', isa => 'Maybe[Int|ArrayRef[Int]]');
@@ -61,29 +66,11 @@ sub BUILD
     shift->reset;
 }
 
-=head2 update
-
-        Update the internal evaluation.
-
-        Parameters
-        ----------
-        labels : list of NDArray
-            The labels of the data.
-
-        preds : list of NDArray
-            Predicted values.
-=cut
-
 method update($label, $pred)
 {
     confess('NotImplemented');
 }
 
-=head2 reset
-
-        Clear the internal statistics to initial state.
-=cut
-
 method reset()
 {
     if(not defined $self->num)
@@ -98,18 +85,6 @@ method reset()
     }
 }
 
-=head2 get
-
-        Get the current evaluation result.
-
-        Returns
-        -------
-        name : str
-           Name of the metric.
-        value : float
-           Value of the evaluation.
-=cut
-
 method get()
 {
     if(not defined $self->num)
@@ -153,11 +128,6 @@ method get_name_value()
     return \%ret;
 }
 
-=head1 DESCRIPTION
-
-    Manage multiple evaluation metrics.
-=cut
-
 package AI::MXNet::CompositeEvalMetric;
 use Mouse;
 
@@ -356,6 +326,7 @@ use AI::MXNet::Base;
 extends 'AI::MXNet::EvalMetric';
 has '+name'        => (default => 'Perplexity');
 has 'ignore_label' => (is => 'ro', isa => 'Maybe[Int]');
+has 'axis'         => (is => 'ro', isa => 'Int', default => -1);
 around BUILDARGS => sub {
     my $orig  = shift;
     my $class = shift;
@@ -370,14 +341,18 @@ AI::MXNet::Perplexity
 
 =head1 DESCRIPTION
 
-    Calculate perplexity
+Calculate perplexity.
 
-    Parameters
-    ----------
-    ignore_label : int or undef
-        index of invalid label to ignore when
-        counting. usually should be -1. Include
-        all entries if undef.
+Parameters
+----------
+ignore_label : int or undef
+    index of invalid label to ignore when
+    counting. usually should be -1. Include
+    all entries if undef.
+axis : int (default -1)
+    The axis from prediction that was used to
+    compute softmax. By default use the last
+    axis.
 =cut
 
 method update(ArrayRef[AI::MXNet::NDArray] $labels, ArrayRef[AI::MXNet::NDArray] $preds)
@@ -393,7 +368,7 @@ method update(ArrayRef[AI::MXNet::NDArray] $labels, ArrayRef[AI::MXNet::NDArray]
             "shape mismatch: (@$label_shape) vs. (@$pred_shape)"
         );
         $label = $label->as_in_context($pred->context)->astype('int32')->reshape([$label->size]);
-        $pred = AI::MXNet::NDArray->batch_take($pred, $label);
+        $pred = AI::MXNet::NDArray->pick($pred, $label, { axis => $self->axis });
         push @{ $probs }, $pred;
     }, $labels, $preds);
 
@@ -525,18 +500,18 @@ method update(ArrayRef[AI::MXNet::NDArray] $labels, ArrayRef[AI::MXNet::NDArray]
 
 =head1 DESCRIPTION
 
-    Custom evaluation metric that takes a NDArray function.
+Custom evaluation metric that takes a sub ref.
 
-    Parameters
-    ----------
-    eval_function : subref
-        Customized evaluation function.
-    name : str, optional
-        The name of the metric
-    allow_extra_outputs : bool
-        If true, the prediction outputs can have extra outputs.
-        This is useful in RNN, where the states are also produced
-        in outputs for forwarding.
+Parameters
+----------
+eval_function : subref
+    Customized evaluation function.
+name : str, optional
+    The name of the metric
+allow_extra_outputs : bool
+    If true, the prediction outputs can have extra outputs.
+    This is useful in RNN, where the states are also produced
+    in outputs for forwarding.
 =cut
 
 
@@ -567,13 +542,13 @@ package AI::MXNet::Metric;
 
 =head2 create
 
-    Create an evaluation metric.
+Create an evaluation metric.
 
-    Parameters
-    ----------
-    metric : str or callable
-        The name of the metric, or a function
-        providing statistics given pred, label NDArray.
+Parameters
+----------
+metric : str or sub ref
+    The name of the metric, or a function
+    providing statistics given pred, label NDArray.
 =cut
 
 my %metrics = qw/
